import dotenv from 'dotenv';
import { ActivityType, Client, Events, GatewayIntentBits, Partials } from 'discord.js'
import axios from 'axios';
import ConfigController from './controller/ConfigController';
import updateCommands from './updateCommands';
import commands from './commands';
import { ServerInfoI } from './types/ServerInfoI';
import ChatController from './controller/ChatController';
import UserController from './controller/UserController';

dotenv.config()

const client = new Client({
  intents: [
    GatewayIntentBits.Guilds,
    GatewayIntentBits.GuildMessages,
    GatewayIntentBits.GuildMessageReactions,
    GatewayIntentBits.GuildMembers
  ],
  partials: [
    Partials.Message,
    Partials.Channel,
    Partials.Reaction
  ],
});

declare global {
  var client: Client<boolean>;
  var config: ConfigController;
}

const config = new ConfigController()
const chatController = new ChatController()
const userController = new UserController()

global.client = client
global.config = config

client.on('ready', () => {
  console.log(`Logged in as ${client.user?.tag}!`);
  updateServerData();
  setInterval(updateServerData, 10000)
  setInterval(() => { chatController.syncMessageWithDiscord() }, 2000)
  setInterval(() => { userController.sync() }, 10000)
});

async function updateServerData() {
  try {
    const { data } = await axios.get<ServerInfoI>(`${process.env.ECO_API_BASE || "https://eco.kamgames.xyz"}/info`, {
      timeout: 6000,
    });
    client.user?.setActivity({
      name: `${data.OnlinePlayers}/${data.TotalPlayers}`,
      type: ActivityType.Playing
    })
    // config.serverInfo = data
  } catch (e) {
    client.user?.setActivity({
      name: `Сервер офлайн`,
      type: ActivityType.Playing
    })
  }
}

client.on(Events.InteractionCreate, async interaction => {
  try {
    if (interaction.isChatInputCommand()) {

      const command = commands.get(interaction.commandName);

      if (!command) {
        console.log(`No command matching ${interaction.commandName} was found.`);
        return;
      }

      try {
        await command.execute(interaction);
      } catch (error) {
        // console.log(`Error executing ${interaction.commandName}`);
        // console.log(error);
        // if (!interaction.deferred) {
        //   await interaction.deferReply({ ephemeral: true })
        // }
        if (!interaction.replied && !interaction.deferred)
          await interaction.reply({ content: "Ошибка выполнения", ephemeral: true })
        else if (interaction.replied || interaction.deferred)
          await interaction.editReply({ content: "Ошибка выполнения" })

        console.log(error)
      }
    } else if (interaction.isAutocomplete()) {
      const command = commands.get(interaction.commandName);

      if (!command) {
        console.error(`No command matching ${interaction.commandName} was found.`);
        return;
      }

      try {
        if (command.autocomplete !== undefined)
          await command.autocomplete(interaction);
      } catch (error) {
        console.error(error);
      }
    }
    return
  } catch (error) {
    // console.log(error)
    console.log(error)
  }
});

client.login(process.env.DISCORD_TOKEN);

updateCommands()
if (!!process.env.DISCORD_TOKEN)
  client.login(process.env.DISCORD_TOKEN);
else
  throw new Error("Token is null");
